#include"TFT_eSPI.h"
TFT_eSPI tft;

// Andy Menon:
// A simple sketch to create an animated face on the LCD Screen 
// of the Seed Wio Terminal.
// Code makes no promises, warranties or guarantees, use at your own risk. 
// Refer to these useful links below to get started.

//Useful Links:
//Seed Wio Terminal - https://www.seeedstudio.com/Wio-Terminal-p-4509.html
//Seed Wiki - https://wiki.seeedstudio.com/Wio-Terminal-Getting-Started/
//Seed Getting Started video: https://www.youtube.com/watch?v=af_I0n5iyzk

// Wio Terminal LCD Display is 320 px wide (X) and 120 px tall (Y)
// In this sketch, the top-left is (0,0) and bottom-right is (320,240)
// X is positive running right to left, Y is positive running top to bottom

// Co-ordinates of the facial features have been initialized here.
// Change them to suit your needs

const int EYE_LINE_Y=60;
const int LEFT_EYE_X=80;
const int RIGHT_EYE_X=240;
const int EYE_WIDTH = 70;
const int EYE_HEIGHT = 30;
const int PUPIL_DIA=31;
const int PUPIL_COLOR=TFT_BLACK;

const int NOSE_LINE_X=160;
const int NOSE_LINE_Y=150;
const int NOSE_X1 =140;
const int NOSE_X2 =180;

const int UPPER_LIP_Y=180;
const int UPPER_LIP_X1=120;
const int UPPER_LIP_X2=200;

const int LOWER_LIP_Y=210;
const int LOWER_LIP_X1=140;
const int LOWER_LIP_X2=180;

void setup() {
    tft.begin();
    tft.setRotation(3);

    // Alternately, color can be specifed as tft.color565(rValue,gValue,bValue) 
    tft.fillScreen(TFT_RED); //Red background
    //Left eye ellipse
    tft.fillEllipse(LEFT_EYE_X,EYE_LINE_Y,EYE_WIDTH,EYE_HEIGHT,TFT_WHITE); 
    //Right eye ellipse
    tft.fillEllipse(RIGHT_EYE_X,EYE_LINE_Y,EYE_WIDTH,EYE_HEIGHT,TFT_WHITE); 
    //Left pupil solid circle
    tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR); 
    //Right pupil solid circle
    tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);

    //Draw nose triangle
    tft.fillTriangle(NOSE_LINE_X, EYE_LINE_Y, NOSE_X1, NOSE_LINE_Y, NOSE_X2, NOSE_LINE_Y,TFT_WHITE);

    //Draw mouth using lines - line functions used here for demo reasons
    //With little creativity, the mouth can be drawn using rectangles and triangles

    //Upper lip line
    tft.drawLine(UPPER_LIP_X1,UPPER_LIP_Y,UPPER_LIP_X2,UPPER_LIP_Y,TFT_WHITE);
    //Lower lip line
    tft.drawLine(LOWER_LIP_X1,LOWER_LIP_Y,LOWER_LIP_X2,LOWER_LIP_Y,TFT_WHITE);
    //Left & right sides of mouth
    tft.drawLine(UPPER_LIP_X1,UPPER_LIP_Y,LOWER_LIP_X1,LOWER_LIP_Y,TFT_WHITE);
    tft.drawLine(UPPER_LIP_X2,UPPER_LIP_Y,LOWER_LIP_X2,LOWER_LIP_Y,TFT_WHITE);
    
}
 
void loop() {

  //Can you Blink?
  Blink(0);
  MoveEyesRight(1);
  MoveEyesLeft(1);
  MakeSquintEyes(1);
 }

 void Blink(int n)
 {
    // if not specified, Blink action will be performed thrice
    if(n<=0){
      n=3;
    }
    for(int i=0;i<n;i++)
    {

      //wipe left pupil
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE); 
      //wipe right pupil
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      delay(250);
      //Restore left pupil
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR); 
      //Restore right pupil
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);
      delay(1000);
      
    }
 }

 void MoveEyesRight(int n)
 {
    // if not specified, action will be performed thrice
    if(n<=0){
      n=3;
    }
    for(int i=0;i<n;i++)
    {
      //wipe both pupils
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE); 
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      //move pupils to right
      tft.fillCircle(LEFT_EYE_X+10,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR); 
      tft.fillCircle(RIGHT_EYE_X+10,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);
      delay(1000);
      //wipe pupils on the right
      tft.fillCircle(LEFT_EYE_X+10,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE); 
      tft.fillCircle(RIGHT_EYE_X+10,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      //move pupils back to center
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR); 
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);
      delay(1000);
    }
 
 }
 void MoveEyesLeft(int n)
 {
    // if not specified, action will be performed thrice
    if(n<=0){
      n=3;
    }
    for(int i=0;i<n;i++)
    {
      //wipe both pupils
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE); 
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      //move pupils to right
      tft.fillCircle(LEFT_EYE_X-10,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR); 
      tft.fillCircle(RIGHT_EYE_X-10,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);
      delay(1000);
      //wipe pupils on the right
      tft.fillCircle(LEFT_EYE_X-10,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE); 
      tft.fillCircle(RIGHT_EYE_X-10,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      //move pupils back to center
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR); 
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);
      delay(1000);
    }
 
 }
  void MakeSquintEyes(int n)
 {
    // if not specified, action will be performed thrice
    if(n<=0){
      n=3;
    }
    for(int i=0;i<n;i++)
    {
      //wipe both pupils in the center
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE); 
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      //move left pupil to right
      tft.fillCircle(LEFT_EYE_X+20,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);      
      //move right pupil to left
      tft.fillCircle(RIGHT_EYE_X-20,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);
      delay(1000);
      //erase pupils in squint position
      tft.fillCircle(LEFT_EYE_X+20,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      tft.fillCircle(RIGHT_EYE_X-20,EYE_LINE_Y,PUPIL_DIA,TFT_WHITE);
      //move pupils back to center
      tft.fillCircle(LEFT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR);
      tft.fillCircle(RIGHT_EYE_X,EYE_LINE_Y,PUPIL_DIA,PUPIL_COLOR); 
      delay(1000);
      
    }
 
 }


